@@ -12,13 +12,15 @@ from typing import Optional
 from typing import NamedTuple
 from typing import TYPE_CHECKING
 
-from liquid.exceptions import LiquidSyntaxError, Error
-from liquid.mode import error
-
 from liquid import expression
+from liquid.expression import IdentifierPathElement
+
 from liquid import ast
 from liquid.stream import TokenStream
 
+from liquid.exceptions import LiquidSyntaxError
+from liquid.exceptions import Error
+
 from liquid.token import TOKEN_ILLEGAL
 from liquid.token import TOKEN_EOF
 from liquid.token import TOKEN_TAG
@@ -71,11 +73,11 @@ if TYPE_CHECKING:
 class Parser:
     """A Liquid template parser. Create a parser tree from a stream of tokens."""
 
-    __slots__ = ("tags", "mode", "illegal", "literal", "statement")
+    __slots__ = ("tags", "illegal", "literal", "statement", "env")
 
     def __init__(self, env: Environment):
         self.tags = env.tags
-        self.mode = env.mode
+        self.env = env
 
         self.illegal = self.tags[TOKEN_ILLEGAL]
         self.literal = self.tags[TOKEN_LITERAL]
@@ -91,13 +93,14 @@ class Parser:
                 if node:
                     root.statements.append(node)
             except Error as err:
-                error(self.mode, err, linenum=stream.current.linenum)
+                self.env.error(err, linenum=stream.current.linenum)
 
             stream.next_token()
 
         return root
 
     def parse_statement(self, stream: TokenStream) -> ast.Node:
+        """Parse a node from a stream of tokens."""
         if stream.current.type == TOKEN_STATEMENT:
             node = self.statement.get_node(stream)
         elif stream.current.type == TOKEN_TAG:
@@ -128,6 +131,8 @@ class Parser:
         return node
 
     def parse_block(self, stream: TokenStream, end: Tuple[str, ...]) -> ast.BlockNode:
+        """Parse multiple nodes from a stream of tokens. Stop parsing nodes when we
+        find a token in `end` or we reach the end of the stream."""
         block = ast.BlockNode(stream.current)
 
         while stream.current.type != TOKEN_EOF:
@@ -156,14 +161,19 @@ def _expect(tok: Token, typ: str, value: Optional[str] = None) -> None:
 
 
 def expect(stream: TokenStream, typ: str, value: Optional[str] = None) -> None:
+    """Raise an excpetion if the current token in the stream does not match the given
+    type and value."""
     _expect(stream.current, typ, value)
 
 
 def expect_peek(stream: TokenStream, typ: str, value: Optional[str] = None) -> None:
+    """Raise an excpetion if the next token in the stream does not match the given
+    type and value."""
     _expect(stream.peek, typ, value)
 
 
 def is_end_tag(tok: Token) -> bool:
+    """Return `True` if the current token looks like an end tag."""
     return tok.value.startswith("end")
 
 
@@ -291,24 +301,22 @@ def parse_identifier(stream: TokenStream) -> expression.Identifier:
 
     while stream.current.type in IDENTIFIER_TOKENS:
         if stream.current.type == TOKEN_IDENTIFIER:
-            path.append(expression.IdentifierPathElement(stream.current.value))
+            path.append(IdentifierPathElement(stream.current.value))
 
         elif stream.current.type == TOKEN_INTEGER:
-            path.append(expression.IdentifierPathElement(int(stream.current.value)))
+            path.append(IdentifierPathElement(int(stream.current.value)))
 
         elif stream.current.type == TOKEN_LBRACKET:
             stream.next_token()  # Eat open bracket
 
             if stream.current.type == TOKEN_STRING:
-                path.append(expression.IdentifierPathElement(stream.current.value))
+                path.append(IdentifierPathElement(stream.current.value))
             elif stream.current.type == TOKEN_NEGATIVE:
                 expect_peek(stream, TOKEN_INTEGER)
                 stream.next_token()
-                path.append(
-                    expression.IdentifierPathElement(-int(stream.current.value))
-                )
+                path.append(IdentifierPathElement(-int(stream.current.value)))
             elif stream.current.type == TOKEN_INTEGER:
-                path.append(expression.IdentifierPathElement(int(stream.current.value)))
+                path.append(IdentifierPathElement(int(stream.current.value)))
 
             elif stream.current.type == TOKEN_IDENTIFIER:
                 # Recursive call to parse_identifier. If it's not a string or
@@ -338,6 +346,9 @@ def parse_string_or_identifier(
     stream: TokenStream,
     linenum: Optional[int] = None,
 ) -> expression.Expression:
+    """Parse an expression from a stream of tokens. If the stream is not at a string or
+    identifier expression, raise a syntax error.
+    """
     if stream.current.type == TOKEN_IDENTIFIER:
         expr: expression.Expression = parse_identifier(stream)
     elif stream.current.type == TOKEN_STRING:
@@ -354,6 +365,8 @@ def parse_unchained_identifier(
     stream: TokenStream,
     linenum: Optional[int] = None,
 ) -> expression.Identifier:
+    """Parse an identifier from a stream of tokens. If the stream is not at an
+    identifier or the identifier is chained, raise a syntax error."""
     expect(stream, TOKEN_IDENTIFIER)
     tok = stream.current
     ident = parse_identifier(stream)
@@ -366,6 +379,9 @@ def parse_unchained_identifier(
     return ident
 
 
+# NOTE: These names are missleading. The args and options in question can apply to any
+# iterable in a loop expression, not just ranges.
+
 RangeArg = Optional[
     Union[
         expression.Identifier,
@@ -377,11 +393,15 @@ RangeArg = Optional[
 
 
 class RangeOption(NamedTuple):
+    """A token and value representing a keyword argument in a loop expression."""
+
     tok: Token
     arg: RangeArg
 
 
 def parse_range_argument(stream: TokenStream) -> RangeArg:
+    """Parse a loop argument value from a stream of tokens. If the stream is not at a
+    valid token for a loop argument value, raise a syntax error."""
     if stream.current.type == TOKEN_IDENTIFIER:
         arg: RangeArg = parse_identifier(stream)
     elif stream.current.type == TOKEN_INTEGER:
@@ -399,6 +419,8 @@ def parse_range_argument(stream: TokenStream) -> RangeArg:
 
 
 def parse_range_option(stream: TokenStream) -> RangeOption:
+    """Parse a loop keyword argument from a stream of tokens. If the stream is not at a
+    valid loop keyword argument, raise a syntax error."""
     tok = stream.current
     if stream.current.type in (TOKEN_LIMIT, TOKEN_OFFSET, TOKEN_COLS):
         expect_peek(stream, TOKEN_COLON)
@@ -572,6 +594,7 @@ class ExpressionParser:
     def parse_prefix_expression(
         self, stream: TokenStream
     ) -> expression.PrefixExpression:
+        """Parse a prefix expression from a stream of tokens."""
         tok = stream.current
         stream.next_token()
 
@@ -587,6 +610,7 @@ class ExpressionParser:
         stream: TokenStream,
         left: expression.Expression,
     ) -> expression.InfixExpression:
+        """Parse an infix expression from a stream of tokens."""
         tok = stream.current
         precedence = self.current_precedence(stream)
         stream.next_token()
@@ -689,10 +713,12 @@ parse_assignment_expression = STANDARD_EXPRESSION_PARSER.parse_assignment_expres
 
 @lru_cache(maxsize=128)
 def get_parser(env: Environment) -> Parser:
+    """Return a template parser for the given environment."""
     return Parser(env)
 
 
 @lru_cache(maxsize=128)
 def get_expression_parser(_: Environment) -> ExpressionParser:
+    """Return an expression parser for the given environment."""
     # Future proofing.
     return STANDARD_EXPRESSION_PARSER
